const std = @import("std");
const microzig = @import("microzig");

const SEGMENT_REMAP: u8 = 0xA1; // 0xA1 = ltr, 0xA0 = rtl
const COM_REMAP: u8 = 0xC8; // 0xC8 = top-down, 0xC0 = bottom-up
const CONTRAST: u8 = 0x7F; // 0x7F
const CHARGE_PERIOD: u8 = 0x22; // 0x22
const MUX_RATIO: u8 = 0x3F; // 0x3F. range 0x0F to 0x3F
const OSC_FREQ: u8 = 0xF0; // LH is divide ratio, UH is oscillator freq, default 0x80

const SCREEN_HEIGHT: u16 = 64;
const SCREEN_WIDTH: u16 = 128;
const SCREEN_SIZE: u16 = SCREEN_HEIGHT * SCREEN_WIDTH / @bitSizeOf(u8);

const rp2040 = microzig.hal;
const i2c = rp2040.i2c;
const gpio = rp2040.gpio;
const led = gpio.num(25);
const time = rp2040.time;
const peripherals = microzig.chip.peripherals;

pub const std_options = struct {
    pub const log_level = .info;
    pub const logFn = rp2040.uart.log;
};

const uart = rp2040.uart.num(0);
const i2c0 = i2c.num(0);

const Screen = struct {
    _buffer: [SCREEN_SIZE + 1]u8 = [_]u8{0x40} ++ [_]u8{0x00} ** SCREEN_SIZE, //0x40 is the "write" command
    fn draw(self: *Screen) void {
        _ = i2c0.write_blocking(@enumFromInt(0x3C), &self._buffer) catch |e| switch (e) {};
    }
    fn fill(self: *Screen, value: u8) void {
        for (self._buffer[1..]) |*v| {
            v.* = value;
        }
    }
    fn apply(self: *Screen, start: struct { x: u7, y: u4 }, data: []const []const u8) void {
        var frame = self._buffer[1..];
        for (0..data.len) |i| {
            const start_frame: usize = SCREEN_WIDTH * (start.y + i) + start.x;
            @memcpy(frame[start_frame .. start_frame + data[0].len], data[i]);
        }
    }
};

var scr = Screen{};

pub fn main() !void {
    setup();

    while (true) {
        // scr.apply(.{ .x = @intCast(i * 8), .y = @intCast(j) }, &[_][]const u8{&[_]u8{0xFF} ** 8} ** 1);
        scr.fill(0x00);
        scr.draw();
        scr.fill(0x55);
        scr.draw();
    }
}

fn setup() void {
    led.set_function(.sio);
    led.set_direction(.out);
    uart.apply(.{
        .baud_rate = 115200,
        .tx_pin = gpio.num(0),
        .rx_pin = gpio.num(1),
        .clock_config = rp2040.clock_config,
    });
    rp2040.uart.init_logger(uart);

    _ = i2c0.apply(.{
        .clock_config = rp2040.clock_config,
        .scl_pin = gpio.num(21),
        .sda_pin = gpio.num(20),
        .baud_rate = 2_000_000,
    });
    led.put(1);

    _ = i2c0.write_blocking(@enumFromInt(0x3C), &[_]u8{ 0xE4, 0xAE, 0xA8, MUX_RATIO, 0x20, 0x00, 0x40, 0xD3, 0x00, SEGMENT_REMAP, COM_REMAP, 0xDA, 0x12, 0x81, CONTRAST, 0xA4, 0xA6, 0xD5, OSC_FREQ, 0x8D, 0x14, 0xD9, CHARGE_PERIOD, 0x2E, 0xAF }) catch unreachable;
    _ = i2c0.write_blocking(@enumFromInt(0x3C), &[_]u8{ 0x20, 0x02, 0x00, 0x10, 0x20, 0x00 }) catch unreachable; // reset pointer
}
